/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

(function(window) {

const LEVELS = [
    {
        task: 'Errate, was dieser Algorithmus bewirkt, ohne ihn auszuprobieren',
        world: '0A13020202000',
        code: 'FALLS istBelegt DANN\n    minus\nvor\nvor\n',
        readonly: true,
        locked: false,
        checkCode: [
            CodeCheckers.reject("Noch nicht ausprobieren")
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ],
        questionWrongFine: 0,
        question: {
            html: "Was bewirkt der Algothmus? (Bei den vorausgewählten Antworten musst du „0“ eintragen, wenn sie nicht zutreffen.)",
            choices: [
                { html: "Der Roboter geht ", unit: "-mal je ein Feld vor", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 2 },
                { html: "Der Roboter hebt ", unit: "-mal je ein Plättchen auf", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 1 },
                { html: "Es gibt eine Notabschaltung" },
                { html: "Der Roboter dreht sich" }
            ]
        }
    },
    {
        task: 'Errate, was derselbe Algorithmus hier bewirkt (wieder, ohne ihn auszuprobieren).',
        world: '0A13000102000',
        code: 'FALLS istBelegt DANN\n    minus\nvor\nvor\n',
        readonly: true,
        locked: false,
        checkCode: [
            CodeCheckers.reject("Noch nicht ausprobieren")
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ],
        questionWrongFine: 0,
        question: {
            html: "Was bewirkt der Algothmus? (Bei den vorausgewählten Antworten musst du „0“ eintragen, wenn sie nicht zutreffen.)",
            choices: [
                { html: "Der Roboter geht ", unit: "-mal je ein Feld vor", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 2 },
                { html: "Der Roboter hebt ", unit: "-mal je ein Plättchen auf", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 0 },
                { html: "Es gibt eine Notabschaltung" },
                { html: "Der Roboter dreht sich" }
            ]
        }
    },
    {
        task: 'Probiere nun den Algorithmus für beide Welten aus.',
        world: ['0A13020202000', '0A13000102000'],
        code: 'FALLS istBelegt DANN\n    minus\nvor\nvor\n',
        readonly: true,
        locked: true,
        start: function() { this._ausprobiert = []; },
        checkCode: [
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            function({ worldDef }) {
                if (!(this._ausprobiert instanceof Array))
                    this._ausprobiert = [];
                if (this._ausprobiert.indexOf(worldDef) == -1)
                    this._ausprobiert.push(worldDef);
                return { accept: true, unlock: (this._ausprobiert.length >= 2) };
            }
        ]
    },
    {
        task: 'Verstanden? Dann kannst du auch diesen Algorithmus interpretieren (wieder, ohne ihn auszuprobieren).',
        world: '0A120000000',
        code: 'FALLS istBelegt DANN\n    vor\nplus\n',
        readonly: true,
        locked: false,
        checkCode: [
            CodeCheckers.reject("Noch nicht ausprobieren")
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ],
        questionWrongFine: 0,
        question: {
            html: "Was bewirkt der Algothmus? (Bei den vorausgewählten Antworten musst du „0“ eintragen, wenn sie nicht zutreffen.)",
            choices: [
                { html: "Der Roboter geht ", unit: "-mal je ein Feld vor", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 0 },
                { html: "Der Roboter hebt ", unit: "-mal je ein Plättchen auf", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 0 },
                { html: "Der Roboter legt ", unit: "-mal je ein Plättchen hin", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 1},
                { html: "Es gibt eine Notabschaltung" },
                { html: "Der Roboter dreht sich" }
            ]
        }
    },
    {
        task: 'Und was bewirkt derselbe Algorithmus hier (wieder, ohne ihn auszuprobieren).',
        world: '0A120101000',
        code: 'FALLS istBelegt DANN\n    vor\nplus\n',
        readonly: true,
        locked: false,
        checkCode: [
            CodeCheckers.reject("Noch nicht ausprobieren")
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            CodeCheckers.accept(true)
        ],
        questionWrongFine: 0,
        question: {
            html: "Was bewirkt der Algothmus? (Bei den vorausgewählten Antworten musst du „0“ eintragen, wenn sie nicht zutreffen.)",
            choices: [
                { html: "Der Roboter geht ", unit: "-mal je ein Feld vor", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 1 },
                { html: "Der Roboter hebt ", unit: "-mal je ein Plättchen auf", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 0 },
                { html: "Der Roboter legt ", unit: "-mal je ein Plättchen hin", correct: true, selectalways: true, type: "number", min: 0, step: 1, accept: 1},
                { html: "Es gibt eine Notabschaltung" },
                { html: "Der Roboter dreht sich" }
            ]
        }
    },
    {
        task: 'Probiere wieder den Algorithmus für beide Welten aus.',
        world: ['0A120000000', '0A120101000'],
        code: 'FALLS istBelegt DANN\n    vor\nplus\n',
        readonly: true,
        locked: true,
        start: function() { this._ausprobiert = []; },
        checkCode: [
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(),
            function({ worldDef }) {
                if (!(this._ausprobiert instanceof Array))
                    this._ausprobiert = [];
                if (this._ausprobiert.indexOf(worldDef) == -1)
                    this._ausprobiert.push(worldDef);
                return { accept: true, unlock: (this._ausprobiert.length >= 2) };
            }
        ]
    }
];

window.roboterCodingTask = {
    levels: LEVELS
};
})(window);
